#include "DataStructures.h"

/*the mother class of all command classes*/

class Command 
{ 
public: 
  virtual bool Execute() = 0; 
  virtual bool Unexecute() = 0;  
};

// The following derived classes correspond to the possible
// "undoable" actions.

// -------------------------------------------------

class UC_ResizeDiameter : public Command
{
public:
	UC_ResizeDiameter(RootflyData *data, int index, int root_id, double radius) {
		m_index = index;
		m_root_id = root_id;
		m_data = data;
		m_new_radius = radius;
	}
	bool Execute();
	bool Unexecute();

private:
	RootflyData *m_data;
	int m_index;
	int m_root_id;
	double m_new_radius;
	double m_old_radius;
};


// -------------------------------------------------

class UC_SetRootAlive : public Command
{
public:
	UC_SetRootAlive(RootflyData *data, int rootid, bool alive) 
	{
		m_data=data; 
		m_rootid=rootid;
		m_alive=alive;

		// need to grab the session that we were on when this was
		// originally called
		ApplicationData *app = m_data->GetApplicationData();
		m_session = app->GetSession();
	}

	bool Execute();
	bool Unexecute();
	
private:
	RootflyData *m_data;
	int m_rootid;
	bool m_alive;
	int m_session;
};


// -------------------------------------------------
class UC_MoveRootSess: public Command
{
public:
  UC_MoveRootSess(RootflyData *data, int root_id, int session, int offset_x, int offset_y) {
	  m_data = data;
	  m_root_id = root_id;
	  m_session = session;
    m_offset_x = offset_x;
	  m_offset_y = offset_y;		
	}
  bool Execute();
	bool Unexecute();

private:
	RootflyData *m_data;
	int m_root_id;
  int m_session;
	double m_offset_x;
	double m_offset_y;
  RootInstance *m_root_inst;

  RootInstance *RootSessGetRootInstance( int m_root_id )
  {
    Experiment *exp = m_data->GetExperiment();
    Tube *tube = exp->GetTube(m_data->GetApplicationData()->GetTube());

    /// Failure if tube is null
    if( tube == NULL ) return false;
  
    Root *r_root = tube->GetRoot(m_root_id);
    /// Failure if root is null
    if( r_root == NULL ) return false;
  
    return r_root->GetRootInstance(m_session);
  } 

  bool RootSessMoveDiameter(RootInstance *root_inst, int index, int root_id, int x, int y)
  {  
    if(root_inst->MoveDiameter(index, x, y)) return true;
    else return false;
  }

  bool RootSessMovePoint(RootInstance *root_inst, int index, int root_id, int x, int y)
  { 
    /// Move the point
    if(root_inst->MovePoint(index, x, y)) return true;
    else return false;  
  }

};


// -------------------------------------------------
class UC_MoveDiameter: public Command
{

public:
  UC_MoveDiameter(RootflyData *data, int index, int root_id, const RPoint& r){
    m_data = data;
    m_index = index;
    m_root_id = root_id;
    m_new_x = r.x;
    m_new_y = r.y;
    m_new_window = r.window;
  }
  bool Execute();
	bool Unexecute();

private:
  RootflyData *m_data;
  int m_index;
  int m_root_id;
  int m_new_x;
  int m_new_y;
  int m_new_window;
  int m_old_x;
  int m_old_y;
  int m_old_window;
  RootInstance *m_root_inst;

  RootInstance *MoveDiameterGetRootInstance (int m_root_id )
  {

    /// Get the current tube and session number from the ApplicationData
    ApplicationData* app_data = m_data->GetApplicationData();
    CString tube_tag = app_data->GetTube();
    int session = app_data->GetSession();
  
    /// Get the tube from the experiment using the tag from the ApplicationData
    Experiment *exp = m_data->GetExperiment();
    Tube* tube = exp->GetTube(tube_tag);
    
     /// Failure if tube is null
    if(tube == NULL)  return NULL;
      
    /// Get the root from the tube by the Root ID
    Root* temp_root = tube->GetRoot(m_root_id);
    
    /// Failure if root is null
    if(temp_root == NULL) return NULL;
    
    /// Get the root instance using the session number from ApplicationData
    return temp_root->GetRootInstance(session);
  }
};


// -------------------------------------------------
class UC_MoveRootsInWindow: public Command
{

public:
  UC_MoveRootsInWindow(RootflyData *data,int window,int offset_x, int offset_y)
  {
    m_data = data;
    m_window = window;
    m_offset_x = offset_x;
    m_offset_y = offset_y;
  }
  bool Execute();
	bool Unexecute();


private:
  RootflyData *m_data;
  int m_window;
  int m_offset_x;
  int m_offset_y;
  int m_session;
  int m_root_id;
  RootInstance *m_root_inst;

  bool RootWindowMoveRootSess(int m_root_id, int session, int m_offset_x, int m_offset_y )
  {
    m_session = session;
    m_root_inst = RootWindowGetRootInstance( m_root_id );
    if( m_root_inst == NULL ) return false;
  
    /// For all points
    for(int i = 0;i<m_root_inst->GetAllPoints()->size();i++)
     RootWindowMovePoint(m_root_inst, i, m_root_id, m_root_inst->GetPoint(i)->x+m_offset_x,m_root_inst->GetPoint(i)->y+m_offset_y);//may need to update this to be a bit different with x and y
  
    /// Move Diameter (assumes only one diameter)
    RootWindowMoveDiameter(m_root_inst, 0,m_root_id, m_root_inst->GetDiameter(0)->r_point.x + m_offset_x, m_root_inst->GetDiameter(0)->r_point.y + m_offset_y);
  
    return true;
  }

  RootInstance *RootWindowGetRootInstance( int m_root_id )
  {
    Experiment *exp = m_data->GetExperiment();
    Tube *tube = exp->GetTube(m_data->GetApplicationData()->GetTube());

    /// Failure if tube is null
    if( tube == NULL ) return false;
  
    Root *r_root = tube->GetRoot(m_root_id);
    /// Failure if root is null
    if( r_root == NULL ) return false;
  
    return r_root->GetRootInstance(m_session);
  } 


  bool RootWindowMoveDiameter(RootInstance *root_inst, int index, int root_id, int x, int y)
  {  
    if(root_inst->MoveDiameter(index, x, y)) return true;
    else return false;
  }

  bool RootWindowMovePoint(RootInstance *root_inst, int index, int root_id, int x, int y)
  { 
    /// Move the point
    if(root_inst->MovePoint(index, x, y)) return true;
    else return false;  
  }
  
};



 
// -------------------------------------------------

class UC_SetRootColor : public Command
{
public:
	UC_SetRootColor(RootflyData *data, int root_id, int color_choice) 
	{
		m_data=data; 
		m_root_id=root_id;
		m_color_choice=color_choice;
	}

	bool Execute();
	bool Unexecute();
	
private:
	RootflyData *m_data;
	int m_root_id;
	int m_color_choice;
	int m_old_color;
};

